/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.photoalbum.manager;

import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.security.Restrict;
import org.richfaces.photoalbum.domain.Album;
import org.richfaces.photoalbum.domain.Image;
import org.richfaces.photoalbum.domain.Shelf;
import org.richfaces.photoalbum.domain.User;
import org.richfaces.photoalbum.service.Constants;
import org.richfaces.photoalbum.util.FileUtils;
import org.richfaces.photoalbum.util.ImageDimension;

@Name("fileManager")
@Scope(ScopeType.APPLICATION)
@AutoCreate
public class FileManager {

    private File uploadRoot;
    
    private String uploadRootPath;
    
    @Create
	public void create() {
		uploadRoot = (File)Component.getInstance(Constants.UPLOAD_ROOT_COMPONENT_NAME, ScopeType.APPLICATION);
		uploadRootPath = (String)Component.getInstance(Constants.UPLOAD_ROOT_PATH_COMPONENT_NAME, ScopeType.APPLICATION);
	}
	
	public void setUploadRoot(String uploadRootPath) throws IOException {
		if (uploadRootPath != null) {
			this.uploadRoot = new File(uploadRootPath);
			for (String f : uploadRoot.list()) {
				File temp = new File(uploadRoot, f);
				temp.delete();
			}
			uploadRoot.delete();
			uploadRoot.mkdirs();
			this.uploadRootPath = this.uploadRoot.getCanonicalPath()
			+ File.separator;
		} else {
			this.uploadRoot = null;
			this.uploadRootPath = null;
		}
	}

    public File getFileByPath(String path) {
        if (this.uploadRoot != null) {
            File result = new File(this.uploadRoot, path);
            try {
                final String resultCanonicalPath = result.getCanonicalPath();
                if (!resultCanonicalPath.startsWith(this.uploadRootPath)) {
                    result = null;
                }
                return result;
            } catch (IOException e) {
                result = null;
            }
            return result;
        }
        return null;
    }

    @Restrict("#{s:hasRole('admin')}")
    @Observer(Constants.ALBUM_DELETED_EVENT)
    public void onAlbumDeleted(Album album, String path) {
    	deleteDirectory(path);
    }
    
    @Restrict("#{s:hasRole('admin')}")
    @Observer(Constants.SHELF_DELETED_EVENT)
    public void onShelfDeleted(Shelf shelf, String path) {
    	deleteDirectory(path);
    }
    
    @Observer(Constants.USER_DELETED_EVENT)
    public void onUserDeleted(User user){
    	deleteDirectory(user.getPath());
    }

    @Observer(Constants.SHELF_ADDED_EVENT)
	public void onShelfAdded(Shelf shelf){
    	File directory = getFileByPath(shelf.getPath());
		FileUtils.addDirectory(directory);
	}
    
    @Observer(Constants.ALBUM_ADDED_EVENT)
	public void onShelfAdded(Album album){
    	File directory = getFileByPath(album.getPath());
		FileUtils.addDirectory(directory);
	}
    
    public boolean saveAvatar(File avatarData, User user) {
    	String avatarPath = File.separator + user.getLogin() + File.separator + Constants.AVATAR_JPG;
		createDirectoryIfNotExist(avatarPath);
		return writeFile(avatarPath, avatarData.getPath(), "", Constants.AVATAR_SIZE, true);
	}

    @Restrict("#{s:hasRole('admin')}")
    @Observer(Constants.IMAGE_DELETED_EVENT)
	public void deleteImage(Image image, String path) {
    	for(ImageDimension d : ImageDimension.values()){
    		FileUtils.deleteFile(getFileByPath(transformPath(path, d.getFilePostfix())));
    	}
	}

    @Restrict("#{s:hasRole('admin')}")
	public boolean addImage(String fileName, String  tempFilePath) {
		createDirectoryIfNotExist(fileName);
		for(ImageDimension d : ImageDimension.values()){
			if(!writeFile(fileName, tempFilePath, d.getFilePostfix(), d.getX(), true)){
				return false;
			}
    	}
		return true;
	}
    
    @Restrict("#{s:hasRole('admin')}")
	public boolean writeFile(String newFileName, String fileName,
			String pattern, int size, boolean includeUploadRoot) {
		BufferedImage bsrc = null;
		try {
			bsrc = FileUtils.bitmapToImage(fileName, Constants.JPG);
		} catch (IOException e1) {
			return false;
		}
		int i = bsrc.getWidth() > bsrc.getHeight() ? bsrc.getWidth() : bsrc.getHeight();
		double d = (double) size / i;
		Double yy = ((Double) d * bsrc.getWidth());
		int width = yy.intValue();
		yy = ((Double) d * bsrc.getHeight());
		int height = yy.intValue();
		// Too small picture or original size
		if (width > bsrc.getWidth() || height > bsrc.getHeight() || size == 0) {
			width = bsrc.getWidth();
			height = bsrc.getHeight();
		}
		BufferedImage bdest = FileUtils.getScaledInstance(bsrc, width, height,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC, true);
		String dest = includeUploadRoot ? this.uploadRootPath
				+ transformPath(newFileName, pattern) : transformPath(
				newFileName, pattern);
		try {
			FileUtils.imageToBitmap(bdest, dest, Constants.JPG);
		} catch (IOException ex) {
			return false;
		}
		return true;
	}

    public String transformPath(String target, String substitute) {
    	if(target.length()<2 || target.lastIndexOf(Constants.DOT) == -1){
    		return "";
    	}
        final String begin = target.substring(0, target.lastIndexOf(Constants.DOT));
        final String end = target.substring(target.lastIndexOf(Constants.DOT));
        return begin + substitute + end;
    }

	public File getFileByAbsolutePath(String path) {
		return new File(path);
	}
	
	public boolean isDirectoryPresent(String path){
		final File file = getFileByPath(path);
		return file.exists() && file.isDirectory();
	}
	
	public boolean isFilePresent(String path){
		final File file = getFileByPath(path);
		return file.exists();
	}
	
	@Observer(Constants.ALBUM_DRAGGED_EVENT)
	public void renameAlbumDirectory(Album album, String pathOld){
		File file = getFileByPath(pathOld);
		File file2 = getFileByPath(album.getPath());
		if(file2.exists()){
			if(file2.isDirectory()){
				FileUtils.deleteDirectory(file2, false);
			}else{
				FileUtils.deleteFile(file2);
			}
		}
		file.renameTo(file2);
	}
	
	@Observer(Constants.IMAGE_DRAGGED_EVENT)
	public void renameImageFile(Image image, String pathOld){
		File file = null;
		File file2 = null;
		for(ImageDimension dimension : ImageDimension.values()){
			file = getFileByPath(transformPath(pathOld, dimension.getFilePostfix()));
			file2 = getFileByPath(transformPath(image.getFullPath(), dimension.getFilePostfix()));
			if(file2.exists()){
				if(file2.isDirectory()){
					FileUtils.deleteDirectory(file2, false);
				}else{
					FileUtils.deleteFile(file2);
				}
			}
			file.renameTo(file2);
		}
	}
	
	private void deleteDirectory(String directory){
    	final File file = getFileByPath(directory);
        FileUtils.deleteDirectory(file, false);
    }
	
	private void createDirectoryIfNotExist(String fileNameNew) {
        final int lastIndexOf = fileNameNew.lastIndexOf(File.separator);
        if (lastIndexOf > 0) {
            final String directory = fileNameNew.substring(0, lastIndexOf);
            final File file = getFileByPath(directory);
            if (!file.exists()) {
                file.mkdirs();
            }
        }
    }
}